@@ -47,6 +47,7 @@ Named parameters:
     code   - coderef or string of shell commands.
     deps   - Tasks this task depends on.
     flags  - list of flags.
+    description - Task description.
 
 =cut
 
@@ -60,6 +61,7 @@ sub new {
         code => $params{ code } || undef,
         deps => $params{ deps } || [],
         flags => $params{ flags } || {},
+        description => $params{ description } || "No Description",
         %{ hook_proto( %params ) },
     }, $class;
 
@@ -174,6 +176,11 @@ sub name {
     return $self->{ name };
 }
 
+sub description {
+    my $self = shift;
+    return $self->{ description };
+}
+
 =back
 =head1 HOOK METHODS AND FUNCTIONS
 
@@ -226,7 +233,7 @@ Chad Granum E<lt>exodist7@gmail.comE<gt>
 
 =head1 COPYRIGHT
 
-Copyright 2008 Chad Granum
+Copyright 2009 Chad Granum
 
 licensed under the GPL version 3.
 You should have received a copy of the GNU General Public License
